/************************************************************************************************/
/*                                                                                              */
/*  Copyright 2010  Broadcom Corporation                                                        */
/*                                                                                              */
/*     Unless you and Broadcom execute a separate written software license agreement governing  */
/*     use of this software, this software is licensed to you under the terms of the GNU        */
/*     General Public License version 2 (the GPL), available at                                 */
/*                                                                                              */
/*          http://www.broadcom.com/licenses/GPLv2.php                                          */
/*                                                                                              */
/*     with the following added to such license:                                                */
/*                                                                                              */
/*     As a special exception, the copyright holders of this software give you permission to    */
/*     link this software with independent modules, and to copy and distribute the resulting    */
/*     executable under terms of your choice, provided that you also meet, for each linked      */
/*     independent module, the terms and conditions of the license of that module.              */
/*     An independent module is a module which is not derived from this software.  The special  */
/*     exception does not apply to any modifications of the software.                           */
/*                                                                                              */
/*     Notwithstanding the above, under no circumstances may you combine this software in any   */
/*     way with any other Broadcom software provided under a license other than the GPL,        */
/*     without Broadcom's express prior written consent.                                        */
/*                                                                                              */
/************************************************************************************************/
#include <linux/kernel.h>
#include <linux/init.h>

#include <mach/pinmux.h>
#include <linux/bug.h>
#include <mach/rdb/brcm_rdb_sysmap.h>
#include <mach/rdb/brcm_rdb_padctrlreg.h>

#define	PIN_DESC(ball, alt1, alt2, alt3, alt4, alt5, alt6)	 	\
	[PN_##ball] = {							\
		.name		=	PN_##ball,			\
		.reg_offset	=	PADCTRLREG_##ball##_OFFSET,	\
		.f_tbl		=	{				\
			PF_##alt1, PF_##alt2, PF_##alt3, 		\
			PF_##alt4, PF_##alt5, PF_##alt6, 		\
		},							\
	}

/*
 * Hawaii chip-level pin description table
 *  generated from
 *     http://confluence.broadcom.com/display/MPSANDRDBLR/Hawaii+pin+description+table+generator
 */
/* Important: please keep this table in pin order defined by enum PIN_NAME */
static const struct pin_desc pin_desc_tbl[PN_MAX] = {
	PIN_DESC(ADCSYN, GPEN09, RESERVED, RESERVED, RESERVED, GPIO36, RESERVED),
	PIN_DESC(BATRM, BATRM, RESERVED, RESERVED, RESERVED, GPIO30, RESERVED),
	PIN_DESC(BSC1CLK, BSC1CLK, RESERVED, RESERVED, RESERVED, GPIO51, RESERVED),
	PIN_DESC(BSC1DAT, BSC1DAT, RESERVED, RESERVED, RESERVED, GPIO52, RESERVED),
	PIN_DESC(CAMCS0, ANA_SYS_REQ1, MMCX_GPO, LCDD16, RESERVED, GPIO43, AP_RSTN),
	PIN_DESC(CAMCS1, CLASSGPWR, ANA_SYS_REQ2, LCDD17, CLK_MON, GPIO44, AP_DAC_DATA_I),
	PIN_DESC(CLK32K, CLK32K, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(CLK_CX8, CLK_CX8, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(DCLK1, DCLK1 , RESERVED, RESERVED, RESERVED, GPIO110, RESERVED),
	PIN_DESC(DCLK4, DCLK4, RESERVED, SSP1DO, PWM4, GPIO95, RESERVED),
	PIN_DESC(DCLKREQ1, DCLKREQ1, RESERVED, RESERVED, RESERVED, GPIO111, RESERVED),
	PIN_DESC(DCLKREQ4, DCLKREQ4, GPEN15, SSP1DI, PWM5, GPIO96, RESERVED),
	PIN_DESC(DMIC0CLK, DMIC0CLK, RESERVED, RESERVED, RESERVED, GPIO123, RESERVED),
	PIN_DESC(DMIC0DQ, DMIC0DQ, RESERVED, RESERVED, RESERVED, GPIO124, RESERVED),
	PIN_DESC(DSI0TE, GPIO37, RESERVED, RESERVED, RESERVED, DSI0TE, RESERVED),
	PIN_DESC(GPIO00, GPIO00, KEY_R0, LCDD15, RESERVED, RESERVED, BBRF_TESTPORT00),
	PIN_DESC(GPIO01, GPIO01, KEY_R1, LCDD14, RESERVED, RESERVED, BBRF_TESTPORT01),
	PIN_DESC(GPIO02, GPIO02, KEY_R2, LCDD13, RESERVED, SSP1DI, BBRF_TESTPORT02),
	PIN_DESC(GPIO03, GPIO03, KEY_R3, LCDD12, RESERVED, SSP1DO, BBRF_TESTPORT03),
	PIN_DESC(GPIO04, GPIO04, KEY_R4, ANA_SYS_REQ3, SIM2RST, RESERVED, BBRF_TESTPORT04),
	PIN_DESC(GPIO05, GPIO05, KEY_R5, ANA_SYS_REQ1, SIM2DAT, RESERVED, BBRF_TESTPORT05),
	PIN_DESC(GPIO06, GPIO06, KEY_R6, RESERVED, RESERVED, SSP2DI, BBRF_TESTPORT06),
	PIN_DESC(GPIO07, GPIO07, KEY_R7, RESERVED, RESERVED, SSP2DO, BBRF_TESTPORT07),
	PIN_DESC(GPIO08, GPIO08, KEY_C0, LCDD11, PWM0, RESERVED, BBRF_TESTPORT08),
	PIN_DESC(GPIO09, GPIO09, KEY_C1, LCDD10, PWM1, RESERVED, BBRF_TESTPORT09),
	PIN_DESC(GPIO10, GPIO10, KEY_C2, LCDD9, PWM2, SSP1CK, BBRF_TESTPORT10),
	PIN_DESC(GPIO11, GPIO11, KEY_C3, LCDD8, RESERVED, SSP1SYN, BBRF_TESTPORT11),
	PIN_DESC(GPIO12, GPIO12, KEY_C4, ANA_SYS_REQ2, T_SHUTDOWN, RESERVED, BBRF_TESTPORT12),
	PIN_DESC(GPIO13, GPIO13, KEY_C5, PWM3, UB3RX, RESERVED, BBRF_TESTPORT13),
	PIN_DESC(GPIO14, GPIO14, KEY_C6, PWM4, RESERVED, SSP2CK, BBRF_TESTPORT14),
	PIN_DESC(GPIO15, GPIO15, KEY_C7, PWM5, RESERVED, SSP2SYN, BBRF_TESTPORT15),
	PIN_DESC(GPIO16, GPIO16, BSC2CLK, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(GPIO17, GPIO17, BSC2DAT, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(GPIO18, GPIO18, LCDCS1, UB2TX, RESERVED, GPEN09, RESERVED),
	PIN_DESC(GPIO19, GPIO19, LCDWE, UB2RX, RESERVED, GPEN10, RESERVED),
	PIN_DESC(GPIO20, GPIO20, LCDRE, UB2RTSN, RESERVED, GPEN11, RESERVED),
	PIN_DESC(GPIO21, GPIO21, LCDD7, UB2CTSN, RESERVED, GPEN12, RESERVED),
	PIN_DESC(GPIO22, GPIO22, LCDD6, PWM0, SSP3CK, RESERVED, RESERVED),
	PIN_DESC(GPIO23, GPIO23, LCDD5, PWM1, SSP3SYN, RESERVED, RESERVED),
	PIN_DESC(GPIO24, GPIO24, LCDD4, PWM2, SSP3DI, RESERVED, RESERVED),
	PIN_DESC(GPIO25, GPIO25, LCDD3, PWM3, SSP3DO, RESERVED, T_SHUTDOWN),
	PIN_DESC(GPIO26, GPIO26, LCDD2, RESERVED, RESERVED, GPEN13, RESERVED),
	PIN_DESC(GPIO27, GPIO27, LCDD1, RESERVED, UB3TX, RESERVED, RESERVED),
	PIN_DESC(GPIO28, GPIO28, RESERVED, RESERVED, RESERVED, GPEN15, RESERVED),
	PIN_DESC(GPIO32, GPIO32, RESERVED, SSP1CK, DCLK2, RESERVED, RESERVED),
	PIN_DESC(GPIO33, GPIO33, RESERVED, SSP1SYN, SYSCLKREQA, DMIC1CLK, RESERVED),
	PIN_DESC(GPIO34, GPIO34, RESERVED, SSP1DO, DCLK3, DMIC1DQ, RESERVED),
	PIN_DESC(GPS_CALREQ, CLASSGPWR, DCLK3, PWM2, RTCK, GPIO99, RESERVED),
	PIN_DESC(GPS_HOSTREQ, RESERVED, SYSCLKREQB, PWM3, GPEN11, GPIO100, SSP1DI),
	PIN_DESC(GPS_PABLANK, RESERVED, SYSCLKREQA, PWM0, GPEN12, GPIO97, RESERVED),
	PIN_DESC(GPS_TMARK, GPEN10, DCLK2, PWM1, RESERVED, GPIO98, RESERVED),
	PIN_DESC(ICUSBDM, USBDN, RESERVED, RESERVED, GPEN11, GPIO122, RESERVED),
	PIN_DESC(ICUSBDP, USBDP, RESERVED, SIM2CLK, AFCPDM, GPIO121, RESERVED),
	PIN_DESC(LCDCS0, LCDCS0 , PC3, RESERVED, RESERVED, GPIO38, RESERVED),
	PIN_DESC(LCDRES, BSC4_CLK, RESERVED, RESERVED, RESERVED, GPIO41, RESERVED),
	PIN_DESC(LCDSCL, BSC3_CLK, LCDCD, RESERVED, RESERVED, GPIO39, LCDSCL),
	PIN_DESC(LCDSDA, BSC3_SDA, LCDD0, RESERVED, RESERVED, GPIO40, LCDSDA),
	PIN_DESC(LCDTE, BSC4_SDA, LCDTE, RESERVED, RESERVED, GPIO42, LCDTE),
	PIN_DESC(MDMGPIO00, RESERVED, RESERVED, GPEN00, RESERVED, GPIO112, RESERVED),
	PIN_DESC(MDMGPIO01, RESERVED, RESERVED, GPEN01, RESERVED, GPIO113, RESERVED),
	PIN_DESC(MDMGPIO02, RESERVED, RESERVED, GPEN02, RESERVED, GPIO114, RESERVED),
	PIN_DESC(MDMGPIO03, RESERVED, RESERVED, GPEN14, RESERVED, GPIO115, RESERVED),
	PIN_DESC(MDMGPIO04, RESERVED, RESERVED, GPEN04, RESERVED, GPIO116, RESERVED),
	PIN_DESC(MDMGPIO05, RESERVED, RESERVED, GPEN05, RESERVED, GPIO117, RESERVED),
	PIN_DESC(MDMGPIO06, RESERVED, RESERVED, GPEN06, RESERVED, GPIO118, RESERVED),
	PIN_DESC(MDMGPIO07, RESERVED, RESERVED, GPEN07, RESERVED, GPIO119, RESERVED),
	PIN_DESC(MDMGPIO08, RESERVED, RESERVED, GPEN08, RESERVED, GPIO120, RESERVED),
	PIN_DESC(MMC0CK, MMC0CK, SSP2CK, RESERVED, RESERVED, GPIO57, RESERVED),
	PIN_DESC(MMC0CMD, MMC0CMD, SSP2DO, RESERVED, RESERVED, GPIO58, RESERVED),
	PIN_DESC(MMC0DAT0, MMC0DAT0, SSP2DI, RESERVED, RESERVED, GPIO67, RESERVED),
	PIN_DESC(MMC0DAT1, MMC0DAT1, RESERVED, RESERVED, RESERVED, GPIO66, RESERVED),
	PIN_DESC(MMC0DAT2, MMC0DAT2, RESERVED, RESERVED, RESERVED, GPIO65, RESERVED),
	PIN_DESC(MMC0DAT3, MMC0DAT3, SSP2SYN, RESERVED, RESERVED, GPIO64, RESERVED),
	PIN_DESC(MMC0DAT4, MMC0DAT4, DCLK3, RESERVED, RESERVED, GPIO63, RESERVED),
	PIN_DESC(MMC0DAT5, MMC0DAT5, DCLK2, RESERVED, RESERVED, GPIO62, RESERVED),
	PIN_DESC(MMC0DAT6, MMC0DAT6, SYSCLKREQB, RESERVED, RESERVED, GPIO61, RESERVED),
	PIN_DESC(MMC0DAT7, MMC0DAT7, SYSCLKREQA, RESERVED, RESERVED, GPIO60, RESERVED),
	PIN_DESC(MMC0RST, MMC0RSTN, RESERVED, RESERVED, RESERVED, GPIO59, RESERVED),
	PIN_DESC(MMC1CK, MMC1CK, PTI_CLK, RESERVED, PWM5, GPIO68, RESERVED),
	PIN_DESC(MMC1CMD, MMC1CMD, RESERVED, RESERVED, PWM4, GPIO69, RESERVED),
	PIN_DESC(MMC1DAT0, MMC1DAT0, PTI_DAT0, RESERVED, RESERVED, GPIO78, RESERVED),
	PIN_DESC(MMC1DAT1, MMC1DAT1, PTI_DAT1, PWM0, RESERVED, GPIO77, RESERVED),
	PIN_DESC(MMC1DAT2, MMC1DAT2, PTI_DAT2, PWM1, RESERVED, GPIO76, RESERVED),
	PIN_DESC(MMC1DAT3, MMC1DAT3, PTI_DAT3, PWM2, RESERVED, GPIO75, RESERVED),
	PIN_DESC(MMC1DAT4, MMC1DAT4, PWM3, SSP2SYN, ANA_SYS_REQ2, GPIO74, SD1DAT3),
	PIN_DESC(MMC1DAT5, MMC1DAT5, PWM4, SSP2DI, ANA_SYS_REQ1, GPIO73, SD1DAT0),
	PIN_DESC(MMC1DAT6, MMC1DAT6, SLEEPCLKTST, SSP2DO, SD1CMD, GPIO72, AP_IHF_DATA_I),
	PIN_DESC(MMC1DAT7, MMC1DAT7, RESERVED, SSP2CK, ANA_SYS_REQ3, GPIO71, SD1CK),
	PIN_DESC(MMC1RST, MMC1RSTN, RESERVED, RESERVED, PWM3, GPIO70, RESERVED),
	PIN_DESC(PC1, PC1, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(PC2, PC2, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(PMBSCCLK, PMBSCCLK, RESERVED, RESERVED, RESERVED, GPIO49, RESERVED),
	PIN_DESC(PMBSCDAT, PMBSCDAT, RESERVED, RESERVED, RESERVED, GPIO50, RESERVED),
	PIN_DESC(PMUINT, GPIO29, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RESETN, RESETN, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RFST2G_MTSLOTEN3G, GPEN03, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RTXDATA2G_TXDATA3G1, RTXDATA2G_TXDATA3G1, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RTXEN2G_TXDATA3G2, RTXEN2G_TXDATA3G2, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RXDATA3G0, RXDATA3G0, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RXDATA3G1, RXDATA3G1, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(RXDATA3G2, RXDATA3G2, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SDCK, SDCK, PTI_CLK, RESERVED, SSP1CK, GPIO79, U1TXD),
	PIN_DESC(SDCMD, SDCMD, UB3RX, RESERVED, SSP1DO, GPIO80, U1RXD),
	PIN_DESC(SDDAT0, SDDAT0, PTI_DAT0, RESERVED, SSP1DI, GPIO84, PM_DEBUG0),
	PIN_DESC(SDDAT1, SDDAT1, PTI_DAT1, RESERVED, SSP1SYN, GPIO83, PM_DEBUG1),
	PIN_DESC(SDDAT2, SDDAT2, PTI_DAT2, RESERVED, SWDIOTMS, GPIO82, PM_DEBUG2),
	PIN_DESC(SDDAT3, SDDAT3, PTI_DAT3, RESERVED, SWCLKTCK, GPIO81, PM_DEBUG3),
	PIN_DESC(SIMCLK, SIMCLK, RESERVED, RESERVED, RESERVED, GPIO55, RESERVED),
	PIN_DESC(SIMDAT, SIMDAT, RESERVED, RESERVED, RESERVED, GPIO54, RESERVED),
	PIN_DESC(SIMDET, SIMDET, RESERVED, RESERVED, RESERVED, GPIO56, RESERVED),
	PIN_DESC(SIMRST, SIMRSTN, RESERVED, RESERVED, RESERVED, GPIO53, RESERVED),
	PIN_DESC(GPIO93, SD1DAT1, GPEN13, SSP1CK, RESERVED, GPIO93, RESERVED),
	PIN_DESC(GPIO94, SD1DAT2, AFCPDM, SSP1SYN, RESERVED, GPIO94, RESERVED),
	PIN_DESC(SPI0CLK, SSP0CK, SD1CK, LCDSDA, DCLK2, GPIO90, AP_SCLK_I),
	PIN_DESC(SPI0FSS, SSP0SYN, SD1DAT3, LCDSCL, SYSCLKREQA, GPIO89, AP_LRCK_I),
	PIN_DESC(SPI0RXD, SSP0DI, SD1DAT0, LCDTE, SYSCLKREQB, GPIO92, AP_REF_2P4M_CLK),
	PIN_DESC(SPI0TXD, SSP0DO, SD1CMD, RESERVED, DCLK3, GPIO91, AP_ADC_DATA_O),
	PIN_DESC(SRI_C, SRI_C, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SRI_D, SRI_D, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SRI_E, SRI_E, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SSPCK, SSP2CK, DMIC1CLK, SIM2CLK, PWM2, GPIO87, RESERVED),
	PIN_DESC(SSPDI, SSP2DI, RESERVED, SIM2DET, PWM3, GPIO88, RESERVED),
	PIN_DESC(SSPDO, SSP2DO, RESERVED, SIM2DAT, PWM1, GPIO86, RESERVED),
	PIN_DESC(SSPSYN, SSP2SYN, DMIC1DQ, SIM2RST, PWM0, GPIO85, RESERVED),
	PIN_DESC(STAT1, STAT1, SIMLDO_EN, RESERVED, RESERVED, GPIO31, RESERVED),
	PIN_DESC(STAT2, STAT2, SIM2LDO_EN, RESERVED, SYSCLKREQB, GPIO35, RESERVED),
	PIN_DESC(SWCLKTCK, SWCLKTCK, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SWDIOTMS, SWDIOTMS, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(SYSCLKEN, SYSCLKEN, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(TDI, TDI, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(TDO, TDO, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(TESTMODE, TESTMODE, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(TRACECLK, TRACECLK, PTI_CLK, RESERVED, DSPTRACECLK, GPIO101, RESERVED),
	PIN_DESC(TRACEDT07, TRACEDT07, UB3RX, RESERVED, DSPTRACEDT07, KEY_R4, RESERVED),
	PIN_DESC(TRACEDT00, TRACEDT00, PTI_DAT0, RESERVED, DSPTRACEDT00, KEY_C7, RESERVED),
	PIN_DESC(TRACEDT01, TRACEDT01, PTI_DAT1, RESERVED, DSPTRACEDT01, KEY_C6, RESERVED),
	PIN_DESC(TRACEDT02, TRACEDT02, PTI_DAT2, RESERVED, DSPTRACEDT02, KEY_C5, RESERVED),
	PIN_DESC(TRACEDT03, TRACEDT03, PTI_DAT3, RESERVED, DSPTRACEDT03, KEY_C4, RESERVED),
	PIN_DESC(TRACEDT04, TRACEDT04, RESERVED, RESERVED, DSPTRACEDT04, KEY_R7, RESERVED),
	PIN_DESC(TRACEDT05, TRACEDT05, RESERVED, RESERVED, DSPTRACEDT05, KEY_R6, RESERVED),
	PIN_DESC(TRACEDT06, TRACEDT06, RESERVED, RESERVED, DSPTRACEDT06, KEY_R5, RESERVED),
	PIN_DESC(TRSTB, TRSTB, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(TXDATA3G0, TXDATA3G0, RESERVED, RESERVED, RESERVED, RESERVED, RESERVED),
	PIN_DESC(UBCTSN, UBCTSN, UB3RX, RESERVED, UB3CTSN, GPIO48, OSC2_OUT),
	PIN_DESC(UBRTSN, UBRTSN, RESERVED, UB3RX, UB3RTSN, GPIO47, OSC1_OUT),
	PIN_DESC(UBRX, UBRX, RESERVED, PWM4, DCLK4, GPIO45, RESERVED),
	PIN_DESC(UBTX, UBTX, RESERVED, PWM5, DCLKREQ4, GPIO46, RESERVED),
	PIN_DESC(TRACEDT08, TRACEDT08, RESERVED, RESERVED, DSPTRACEDT08, KEY_C3, RESERVED),
	PIN_DESC(TRACEDT09, TRACEDT09, RESERVED, RESERVED, DSPTRACEDT09, KEY_C2, RESERVED),
	PIN_DESC(TRACEDT10, TRACEDT10, RESERVED, RESERVED, DSPTRACEDT10, KEY_C1, RESERVED),
	PIN_DESC(TRACEDT11, TRACEDT11, RESERVED, RESERVED, DSPTRACEDT11, KEY_C0, RESERVED),
	PIN_DESC(TRACEDT12, TRACEDT12, RESERVED, RESERVED, DSPTRACEDT12, KEY_R3, RESERVED),
	PIN_DESC(TRACEDT13, TRACEDT13, RESERVED, RESERVED, DSPTRACEDT13, KEY_R2, RESERVED),
	PIN_DESC(TRACEDT14, TRACEDT14, RESERVED, RESERVED, DSPTRACEDT14, KEY_R1, RESERVED),
	PIN_DESC(TRACEDT15, TRACEDT15, RESERVED, RESERVED, DSPTRACEDT15, KEY_R0, RESERVED),
};

struct chip_pin_desc g_chip_pin_desc = {
	.desc_tbl = pin_desc_tbl,
};

/* unlock access lock */
static void __init hawaii_unlock_padctl(void __iomem *base)
{
#define PASSWORD 0xA5A501
	writel(PASSWORD, base + PADCTRLREG_WR_ACCESS_OFFSET);
	writel(0x0, base + PADCTRLREG_ACCESS_LOCK0_OFFSET);

	writel(PASSWORD, base + PADCTRLREG_WR_ACCESS_OFFSET);
	writel(0x0, base + PADCTRLREG_ACCESS_LOCK1_OFFSET);

	writel(PASSWORD, base + PADCTRLREG_WR_ACCESS_OFFSET);
	writel(0x0, base + PADCTRLREG_ACCESS_LOCK2_OFFSET);

	writel(PASSWORD, base + PADCTRLREG_WR_ACCESS_OFFSET);
	writel(0x0, base + PADCTRLREG_ACCESS_LOCK3_OFFSET);

	writel(PASSWORD, base + PADCTRLREG_WR_ACCESS_OFFSET);
	writel(0x0, base + PADCTRLREG_ACCESS_LOCK4_OFFSET);
}

int __init pinmux_chip_init(void)
{
	g_chip_pin_desc.base = ioremap(PAD_CTRL_BASE_ADDR, SZ_2K);
	BUG_ON(!g_chip_pin_desc.base);

	hawaii_unlock_padctl(g_chip_pin_desc.base);
	return 0;
}

/* Maitained for compatibility. Remove. */
int __init pinmux_board_init(void)
{
	return 0;
}
