//***************************************************************************
//
// (c)1999-2011 Broadcom Corporation
//
// Unless you and Broadcom execute a separate written software license agreement governing use of this software,
// this software is licensed to you under the terms of the GNU General Public License version 2,
// available at http://www.broadcom.com/licenses/GPLv2.php (the "GPL").
//
//***************************************************************************
/**
*
*   @file   audio_tuning.h
*
*   @brief  This file contains constants related to audio tuning.
*
****************************************************************************/

#ifndef _INC_AUDIO_TUNING_H_
#define _INC_AUDIO_TUNING_H_

/****************************************************************
 * The following part shows all the constant values needed in audio baseband code.
 ****************************************************************/

/* Define whether to suppor thew new audio sysparm data structure
 * This new structure is to reduce the memory consumption of the
 * audio sysparm. It will remove the DSP sysparm data from some
 * audio profiles which do not use DSP
 * When commenting it out, the new audio sysparm data structure
 * is disabled.
 */
#define USE_NEW_AUDIO_MM_PARAM

/* Number of audio modes */
#define	AUDIO_MODE_NUMBER	9

#if defined(USE_NEW_AUDIO_MM_PARAM)
/* Number of audio profiles for voice call or voice processing */
#define AUDIO_APP_NUMBER        12
/* Number of audio profiles for multimedia. No DSP involved */
#define AUDIO_APP_MM_NUMBER     6
#else
/* Number of audio profiles for voice call or voice processing */
#define AUDIO_APP_NUMBER        16
#endif

#define AUDIO_MODE_NUMBER_VOICE	(AUDIO_MODE_NUMBER*AUDIO_APP_NUMBER)

#define NUM_OF_ENTRY_IN_DSP_VOICE_VOLUME_TABLE		15
#define NUM_OF_ENTRY_IN_FM_RADIO_DIGITAL_VOLUME		15

typedef enum  {
	PARAM_AUDIO_AGC_DL_DECAY,	/* 0  to delete */
	PARAM_AUDIO_AGC_DL_ENABLE,	/* 1 */
	PARAM_AUDIO_AGC_DL_HI_THRESH,	/* 2 */
	PARAM_AUDIO_AGC_DL_LOW_THRESH,	/* 3 */
	PARAM_AUDIO_AGC_DL_MAX_IDX,	/* 4 */
	PARAM_AUDIO_AGC_DL_MAX_STEP_DOWN,	/* 5 */
	PARAM_AUDIO_AGC_DL_MAX_THRESH,	/* 6 */
	PARAM_AUDIO_AGC_DL_MIN_IDX,	/* 7 */
	PARAM_AUDIO_AGC_DL_STEP_DOWN,	/* 8 */
	PARAM_AUDIO_AGC_DL_STEP_UP,	/* 9 */
	PARAM_AUDIO_AGC_UL_DECAY,	/* 10 */
	PARAM_AUDIO_AGC_UL_ENABLE,	/* 11 */
	PARAM_AUDIO_AGC_UL_HI_THRESH,	/* 12 */
	PARAM_AUDIO_AGC_UL_LOW_THRESH,	/* 13 */
	PARAM_AUDIO_AGC_UL_MAX_IDX,	/* 14 */
	PARAM_AUDIO_AGC_UL_MAX_STEP_DOWN,	/* 15 */
	PARAM_AUDIO_AGC_UL_MAX_THRESH,	/* 16 */
	PARAM_AUDIO_AGC_UL_MIN_IDX,	/* 17 */
	PARAM_AUDIO_AGC_UL_STEP_DOWN,	/* 18 */
	PARAM_AUDIO_AGC_UL_STEP_UP,	/* 19 */
	PARAM_ECHO_ADAPT_NORM_FACTOR,	/* 20 */
	PARAM_ECHO_CANCEL_DTD_HANG,	/* 21 */
	PARAM_ECHO_CANCEL_DTD_THRESH,	/* 22 */
	PARAM_ECHO_CANCEL_FRAME_SAMPLES,	/* 23 */
	PARAM_ECHO_CANCEL_HSEC_LOOP,	/* 24 */
	PARAM_ECHO_CANCEL_HSEC_MFACT,	/* 25 */
	PARAM_ECHO_CANCEL_HSEC_STEP,	/* 26 */
	PARAM_ECHO_CANCEL_INPUT_GAIN,	/* 27 */
	PARAM_ECHO_CANCEL_MAX_HSEC,	/* 28 */
	PARAM_ECHO_CANCEL_OUTPUT_GAIN,	/* 29 */
	PARAM_ECHO_CANCEL_UPDATE_DELAY,	/* 30 */
	PARAM_ECHO_CANCELLING_ENABLE,	/* 31 */
	PARAM_ECHO_CNG_BIAS,	/* 32 */
	PARAM_ECHO_CNG_ENABLE,	/* 33 */
	PARAM_ECHO_COUPLING_DELAY,	/* 34 */
	PARAM_ECHO_DIGITAL_INPUT_CLIP_LEVEL,	/* 35 */
	PARAM_ECHO_DUAL_FILTER_MODE,	/* 36 */
	PARAM_ECHO_EN_FAR_SCALE_FACTOR,	/* 37 */
	PARAM_ECHO_EN_NEAR_SCALE_FACTOR,	/* 38 */
	PARAM_ECHO_FAR_IN_FILTER,	/* 39 */
	PARAM_ECHO_FEED_FORWARD_GAIN,	/* 40 */
	PARAM_ECHO_NLP_CNG_FILTER,	/* 41 */
	PARAM_ECHO_NLP_DL_ENERGY_WINDOW,	/* 42 */
	PARAM_ECHO_NLP_DL_GAIN_TABLE,	/* 43 */
	PARAM_ECHO_NLP_DL_IDLE_UL_GAIN,	/* 44 */
	PARAM_ECHO_NLP_DOWNLINK_VOLUME_CTRL,	/* 45 */
	PARAM_ECHO_NLP_DTALK_DL_GAIN,	/* 46 */
	PARAM_ECHO_NLP_DTALK_UL_GAIN,	/* 47 */
	PARAM_ECHO_NLP_ENABLE,	/* 48 */
	PARAM_ECHO_NLP_GAIN,	/* 49 */
	PARAM_ECHO_NLP_MAX_SUPP,	/* 50 */
	PARAM_ECHO_NLP_MIN_DL_PWR,	/* 51 */
	PARAM_ECHO_NLP_MIN_UL_PWR,	/* 52 */
	PARAM_ECHO_NLP_RELATIVE_DL_ENERGY_DECAY,	/* 53 */
	PARAM_ECHO_NLP_RELATIVE_DL_ENERGY_WINDOW,	/* 54 */
	PARAM_ECHO_NLP_RELATIVE_DL_W_THRESH,	/* 55 */
	PARAM_ECHO_NLP_RELATIVE_OFFSET_DL_ACTIVE,	/* 56 */
	PARAM_ECHO_NLP_RELATIVE_OFFSET_DTALK,	/* 57 */
	PARAM_ECHO_NLP_TIMEOUT_VAL,	/* 58 */
	PARAM_ECHO_NLP_UL_ACTIVE_DL_GAIN,	/* 59 */
	PARAM_ECHO_NLP_UL_BRK_IN_THRESH,	/* 60 */
	PARAM_ECHO_NLP_UL_ENERGY_WINDOW,	/* 61 */
	PARAM_ECHO_NLP_UL_GAIN_TABLE,	/* 62 */
	PARAM_ECHO_NLP_UL_IDLE_DL_GAIN,	/* 63 */
	PARAM_ECHO_NLP_DTALK_HANG_COUNT,	/* 64 */
	PARAM_ECHO_NLP_UL_ACTIVE_HANG_COUNT,	/* 65 */
	PARAM_ECHO_NLP_RELATIVE_OFFSET_UL_ACTIVE,	/* 66 */
	PARAM_ECHO_SPKR_PHONE_INPUT_GAIN,	/* 67 */
	PARAM_ECHO_STABLE_COEF_THRESH,	/* 68 */
	PARAM_MIC_PGA,		/* 69 */
	PARAM_SPEAKER_PGA,	/* 70 */
	PARAM_UL_NOISE_SUPP_INPUT_GAIN,	/* 71 */
	PARAM_UL_NOISE_SUPP_OUTPUT_GAIN,	/* 72 */
	PARAM_UL_NOISE_SUPPRESSION_ENABLE,	/* 73 */
	PARAM_SIDETONE,		/* 74 */
	PARAM_VOICE_ADC,	/* 75 */
	PARAM_VOICE_DAC,	/* 76 */
	PARAM_AUDIO_CHANNEL,	/* 77   to delete */
	/* PARAM_AUDIO_DEVICE_TYPE, */
	PARAM_NOISE_SUPP_MIN = 79,	/* 79 */
	PARAM_UL_NOISE_SUPP_MAX,	/* 80 */
	PARAM_VOLUME_STEP_SIZE,	/* 81 */
	PARAM_NUM_SUPPORTED_VOLUME_LEVELS,	/* 82 */
	PARAM_DAC_FILTER_SCALE_FACTOR,	/* 83   to delete */

	PARAM_ECHO_DUAL_EC_ECLEN = 84,	/* 84 */
	PARAM_ECHO_DUAL_EC_DT_TH_ERL_dB = 87,	/* 87 */
	PARAM_ECHO_DUAL_EC_echo_step_size_gain,	/* 88 */

	PARAM_COMPANDER_FLAG = 92,	/* 92 */
	PARAM_EXPANDER_ALPHA,		/* 93 */
	PARAM_EXPANDER_BETA,		/* 94 */
	PARAM_EXPANDER_UPPER_LIMIT,	/* 95 */
	PARAM_EXPANDER_C,		/* 96 */
	PARAM_COMPRESSOR_ALPHA,		/* 97 */
	PARAM_COMPRESSOR_BETA,		/* 98 */
	PARAM_COMPRESSOR_OUTPUT_GAIN,	/* 99 */
	PARAM_COMPRESSOR_GAIN,		/* 100 */

	PARAM_COMPANDER_FLAG_UL,	/* 101 */
	PARAM_EXPANDER_ALPHA_UL,	/* 102 */
	PARAM_EXPANDER_BETA_UL,		/* 103 */
	PARAM_EXPANDER_UPPER_LIMIT_UL,	/* 104 */
	PARAM_EXPANDER_C_UL,		/* 105 */
	PARAM_COMPRESSOR_ALPHA_UL,	/* 106 */
	PARAM_COMPRESSOR_BETA_UL,	/* 107 */
	PARAM_COMPRESSOR_OUTPUT_GAIN_UL,	/* 108 */
	PARAM_COMPRESSOR_GAIN_UL,	/* 109 */

	PARAM_AUDIO_DSP_SIDETONE,	/* 110 */
	PARAM_AUDIO_DL_IDLE_PGA_ADJ,	/* 111 */
	PARAM_AUDIO_NS_UL_IDLE_ADJ,	/* 112 */

	PARAM_SIDETONE_OUTPUT_GAIN,	/* 113 */
	PARAM_SIDETONE_BIQUAD_SCALE_FACTOR,	/* 114 */
	PARAM_SIDETONE_BIQUAD_SYS_GAIN,	/* 115 */

	PARAM_AUDIO_EC_DE_EMP_FILT_COEF,	/* 116 */
	PARAM_AUDIO_EC_PRE_EMP_FILT_COEF,	/* 117 */

	PARAM_AUDIO_HPF_ENABLE,		/* 118 */
	PARAM_AUDIO_UL_HPF_COEF_B,	/* 119 */
	PARAM_AUDIO_UL_HPF_COEF_A,	/* 120 */
	PARAM_AUDIO_DL_HPF_COEF_B,	/* 121 */
	PARAM_AUDIO_DL_HPF_COEF_A,	/* 122 */

	PARAM_AUDVOC_VCFGR,		/* 123 */


	/*  PARAM_BLUETOOTH_FILTER_ENABLE, */      /* 124 */
	/*  PARAM_BLUETOOTH_UL_BIQUAD_GAIN, */     /* 125 */
	/*  PARAM_BLUETOOTH_DL_BIQUAD_GAIN, */    /* 126 */
	/*  PARAM_BLUETOOTH_DL_FILTER, */         /* 127 */
	/*  PARAM_BLUETOOTH_UL_FILTER,  */        /* 128 */


	/*  PARAM_COMP_BIQUAD_GAIN,   */          /* 129 */
	/*  PARAM_COMP_BIQUAD_FILTER,  */         /* 130 */
	PARAM_ECHO_SUBBAND_NLP_DISTORTION_THRESH = 131,
	PARAM_ECHO_SUBBAND_NLP_UL_MARGIN,	/* 132 */
	PARAM_ECHO_SUBBAND_NLP_NOISE_MARGIN,	/* 133 */

	PARAM_DL_SUBBAND_COMPANDER_FLAG,	/* 134 */



    /* 3-B Compr parameters are no more used */
	/*PARAM_COMPRESS_DL_FIR1_T2LIN, */		  /* 135 */
    /*PARAM_COMPRESS_DL_FIR1_G2T2, */		  /* 136 */
    /*PARAM_COMPRESS_DL_FIR1_G3T3, */		  /* 137 */
    /*PARAM_COMPRESS_DL_FIR1_G4T4, */		  /* 138 */
    /*PARAM_COMPRESS_DL_FIR1_ALPHA,*/		  /* 139 */
    /*PARAM_COMPRESS_DL_FIR1_BETA, */		  /* 140 */
    /*PARAM_COMPRESS_DL_FIR1_G1LIN,	*/		  /* 141 */
    /*PARAM_COMPRESS_DL_FIR1_STEP2,	*/		  /* 142 */
    /*PARAM_COMPRESS_DL_FIR1_STEP3,	*/		  /* 143 */
    /*PARAM_COMPRESS_DL_FIR1_STEP4,	*/		  /* 144 */

	/*PARAM_COMPRESS_DL_FIR2_T2LIN,	*/		  /* 145 */
	/*PARAM_COMPRESS_DL_FIR2_G2T2, */		  /* 146 */
	/*PARAM_COMPRESS_DL_FIR2_G3T3, */		  /* 147 */
	/*PARAM_COMPRESS_DL_FIR2_G4T4, */		  /* 148 */
	/*PARAM_COMPRESS_DL_FIR2_ALPHA,*/		  /* 149 */
	/*PARAM_COMPRESS_DL_FIR2_BETA, */		  /* 150 */
	/*PARAM_COMPRESS_DL_FIR2_G1LIN,	*/		  /* 151 */
	/*PARAM_COMPRESS_DL_FIR2_STEP2,	*/		  /* 152 */
	/*PARAM_COMPRESS_DL_FIR2_STEP3,	*/		  /* 153 */
	/*PARAM_COMPRESS_DL_FIR2_STEP4,	*/		  /* 154 */


    /*PARAM_COMPRESS_DL_STREAM3_T2LIN,*/	   /* 155 */
    /*PARAM_COMPRESS_DL_STREAM3_G2T2, */	   /* 156 */
    /*PARAM_COMPRESS_DL_STREAM3_G3T3, */	   /* 157 */
    /*PARAM_COMPRESS_DL_STREAM3_G4T4, */	   /* 158 */
    /*PARAM_COMPRESS_DL_STREAM3_ALPHA, */	   /* 159 */
    /*PARAM_COMPRESS_DL_STREAM3_BETA,  */	   /* 160 */
    /*PARAM_COMPRESS_DL_STREAM3_G1LIN, */	   /* 161 */
    /*PARAM_COMPRESS_DL_STREAM3_STEP2, */	   /* 162 */
    /*PARAM_COMPRESS_DL_STREAM3_STEP3, */	   /* 163 */
    /*PARAM_COMPRESS_DL_STREAM3_STEP4, */	   /* 164 */

	PARAM_COMPRESS_DL_T2LIN = 165,	/* 165 */
	PARAM_COMPRESS_DL_G2T2,	/* 166 */
	PARAM_COMPRESS_DL_G3T3,	/* 167 */
	PARAM_COMPRESS_DL_G4T4,	/* 168 */
	PARAM_COMPRESS_DL_ALPHA,	/* 169 */
	PARAM_COMPRESS_DL_BETA,	/* 170 */
	PARAM_COMPRESS_DL_G1LIN,	/* 171 */
	PARAM_COMPRESS_DL_STEP2,	/* 172 */
	PARAM_COMPRESS_DL_STEP3,	/* 173 */
	PARAM_COMPRESS_DL_STEP4,	/* 174 */

	PARAM_ECHO_SUBBAND_NLP_ENABLE,	/* 175 */

	PARAM_VOICE_VOLUME_MAX,	/* 176 */
	PARAM_VOICE_VOLUME_INIT,	/* 177 */
	PARAM_AUDVOC_ADAC_IIR,	/* 178  (5 Biquads, total 25 coeff) */
	PARAM_AUDVOC_AEQMODE,	/* 179 */
	PARAM_AUDVOC_AEQPATHGAIN,	/* 180 */
	PARAM_AUDVOC_AEQCOF,	/* 181 */
	PARAM_AUDVOC_MIXER_IIR,	/* 182 */

	PARAM_MPM_NIIR_COEFFICIENT,	/* 183 */
	PARAM_MPM_GAIN_ATTACK_STEP,	/* 184 */
	PARAM_MPM_GAIN_ATTACK_SLOPE,	/* 185 */
	PARAM_MPM_GAIN_DECAY_SLOPE,	/* 186 */
	PARAM_MPM_GAIN_ATTACK_THRESHOLD,	/* 187 */
	PARAM_IHF_PROTECTION_ENABLE,	/* 188 */
	PARAM_GE_CENTER_FREQ_1,	/* 189 */
	PARAM_GE_CENTER_FREQ_2,	/* 190 */
	PARAM_GE_CENTER_FREQ_3,	/* 191 */
	PARAM_GE_CENTER_FREQ_4,	/* 192 */
	PARAM_GE_CENTER_FREQ_5,	/* 193 */
	PARAM_GE_BANDWIDTH_1,	/* 194 */
	PARAM_GE_BANDWIDTH_2,	/* 195 */
	PARAM_GE_BANDWIDTH_3,	/* 196 */
	PARAM_GE_BANDWIDTH_4,	/* 197 */
	PARAM_GE_BANDWIDTH_5,	/* 198 */
	PARAM_GE_CROSSOVER_FREQ_1,	/* 199 */
	PARAM_GE_CROSSOVER_FREQ_2,	/* 200 */
	PARAM_GE_CROSSOVER_FREQ_3,	/* 201 */
	PARAM_GE_CROSSOVER_FREQ_4,	/* 202 */

	PARAM_SIDETONE_FILTER,	/* 203 */
	/*parm used for Omega Voice tuning */
	PARAM_OMEGA_VOICE_ENABLE = 204,	/* 204 */
	PARAM_OMEGA_VOICE_THRES_DB = 205,	/* 205 */
	PARAM_OMEGA_VOICE_MAX_GAIN_DB,	/* 206 */
	PARAM_OMEGA_VOICE_GAIN_STEP_UP_DB256,	/* 207 */
	PARAM_OMEGA_VOICE_GAIN_STEP_DN_DB256,	/* 208 */
	PARAM_OMEGA_VOICE_MAX_GAIN_DB_SCALE_FACTOR,	/* 209 */

	PARAM_VOICE_DAC_HPF_ENABLE = 210,	/* 210 */
	PARAM_VOICE_DAC_HPF_CUTOFF_FREQ,	/* 211 */
    /***
    PARAM_AUDVOC_ADAC_IIR_HPF_ENABLE=212,
    PARAM_AUDVOC_ADAC_IIR_HPF_CUTOFF_FREQ,
    ***/
	PARAM_OMEGA_VOICE_MAX_ALLOWED_GAIN_SPREAD_DB = 214,	/* 214 */

	PARAM_AUDVOC_MIXER_IIR_HPF_ENABLE = 215,	/* 215 */
	PARAM_AUDVOC_MIXER_IIR_HPF_CUTOFF_FREQ = 216,	/* 216 */
	PARAM_MPMBIQUAD_CFG = 217,	/* 217 */

	PARAM_EXT_SPEAKER_PGA,	/* 218 */
	PARAM_EXT_SPEAKER_PREAMP_PGA,	/* 219 */

	/*  PARAM_BLUETOOTH_DL_BIQUAD_OUTPUT_GAIN = 220, */
	/*  PARAM_BLUETOOTH_UL_BIQUAD_OUTPUT_GAIN,   */     /* 221 */

	PARAM_ALPHA_VOICE_ENABLE = 222,	/* 222 */
	PARAM_ALPHA_VOICE_AVC_TARGET_DB,	/* 223 */
	PARAM_ALPHA_VOICE_AVC_MAX_GAIN_DB,	/* 224 */
	PARAM_ALPHA_VOICE_AVC_STEP_UP_DB256,	/* 225 */
	PARAM_ALPHA_VOICE_AVC_STEP_DOWN_DB256,	/* 226 */

	/* KAPPA_VOICE */
	PARAM_KAPPA_VOICE_ENABLE,	/* 227 */
	PARAM_KAPPA_VOICE_HIGH_PITCHED_TONE_TARGET_DB,	/* 228 */
	PARAM_KAPPA_VOICE_FAST_DETECTOR_LOUDNESS_TARGET_DB,	/* 229 */
	PARAM_KAPPA_VOICE_FAST_DETECTOR_MAX_SUPP_DB,	/* 230 */
	PARAM_KAPPA_VOICE_SLOW_DETECTOR_LOUDNESS_TARGET_DB,	/* 231 */
	PARAM_KAPPA_VOICE_SLOW_DETECTOR_MAX_SUPP_DB,	/* 232 */

	PARAM_EXT_SPEAKER_ENABLE = 233,	/* 233  (not used in Athena) */
	PARAM_ECHO_SUBBAND_NLP_ERL_ERLE_ADJ_WB = 234,	/* not used in Athena */
	PARAM_ECHO_SUBBAND_NLP_UL_MARGIN_WB,	/* not used in Athena */
	PARAM_ECHO_SUBBAND_NLP_NOISE_MARGIN_WB,	/* not used in Athena */

	PARAM_VOICE_DUALMIC_ENABLE = 237,	/* 237 */
	PARAM_VOICE_ADC_D1,	/* 238 */
	PARAM_VOICE_ADC_D2,	/* 239 */
	PARAM_VOICE_DIGMIC_SAMPLE_SEL,	/* 240 */

	PARAM_ECHO_SUBBAND_NLP_DT_FINE_CONTROL,	/* 241 */

	PARAM_VOICE_DUALMIC_ANC_ENABLE = 242,	/* 242 */
	PARAM_VOICE_DUALMIC_NLP_ENABLE,		/* 243 */
	PARAM_VOICE_DUALMIC_PRE_WNR_ENABLE,	/* 244 */
	PARAM_VOICE_DUALMIC_POST_WNR_ENABLE,	/* 245 */
	PARAM_VOICE_DUALMIC_NS_LEVEL,		/* 246 */

	PARAM_ECHO_CANCEL_MIC2_INPUT_GAIN = 247,	/* 247 */
	PARAM_ECHO_CANCEL_MIC2_OUTPUT_GAIN,	/* 248 */
	PARAM_ECHO_MIC2_FEED_FORWARD_GAIN,	/* 249 */

	PARAM_NLP_DISTORTION_COUPLING = 250,	/* 250 */

	PARAM_AMIC_DGA_COARSE_GAIN = 251,	/*251 */
	PARAM_AMIC_DGA_FINE_GAIN,	/*252 */
	PARAM_DMIC1_DGA_COARSE_GAIN,	/*253 */
	PARAM_DMIC1_DGA_FINE_GAIN,	/*254 */
	PARAM_DMIC2_DGA_COARSE_GAIN,	/*255 */
	PARAM_DMIC2_DGA_FINE_GAIN,	/*256 */
	PARAM_DMIC3_DGA_COARSE_GAIN,	/*257 */
	PARAM_DMIC3_DGA_FINE_GAIN,	/*258 */
	PARAM_DMIC4_DGA_COARSE_GAIN,	/*259 */
	PARAM_DMIC4_DGA_FINE_GAIN,	/*260 */

	PARAM_HW_MIXER_INPUT_GAIN_CTRL_L,	/* 261 */
	PARAM_HW_MIXER_COARSE_GAIN_CTRL_L,	/* 262 */
	PARAM_HW_MIXER_FINE_GAIN_CTRL_L,	/* 263 */
	PARAM_HW_MIXER_INPUT_GAIN_CTRL_R,	/* 264 */
	PARAM_HW_MIXER_COARSE_GAIN_CTRL_R,	/* 265 */
	PARAM_HW_MIXER_FINE_GAIN_CTRL_R,	/* 266    */

	PARAM_PMU_HIGH_GAIN_MODE_FLAG,	/* 267 */
	PARAM_PMU_SPEAKER_PGA_LEFT_CHANNEL,	/* 268 */
	PARAM_PMU_SPEAKER_PGA_RIGHT_CHANNEL,	/* 269 */

	PARAM_VOICE_MIC1_BIQUAD_NUM,	/* 270 */
	PARAM_VOICE_MIC1_SCALE_INPUT,	/* 271 */
	PARAM_VOICE_MIC1_SCALE_OUTPUT,	/* 272 */
	PARAM_VOICE_MIC1_EQ,	/* 273 */

	PARAM_VOICE_MIC2_BIQUAD_NUM,	/* 274 */
	PARAM_VOICE_MIC2_SCALE_INPUT,	/* 275 */
	PARAM_VOICE_MIC2_SCALE_OUTPUT,	/* 276 */
	PARAM_VOICE_MIC2_EQ,	/* 277 */

	PARAM_VOICE_SPEAKER_BIQUAD_NUM,	/* 278 */
	PARAM_VOICE_SPEAKER_SCALE_INPUT,	/* 279 */
	PARAM_VOICE_SPEAKER_SCALE_OUTPUT,	/* 280 */
	PARAM_VOICE_SPEAKER_EQ,	/* 281 */
	PARAM_VOICE_SPEAKER_HPF_ENABLE,	/* 282 */
	PARAM_VOICE_SPEAKER_HPF_CUTOFF_FREQ,	/* 283 */

	/* Reserved */	/* 284 */
	/* Reserved */	/* 285 */
	/* Reserved */	/* 286 */
	/* Reserved */	/* 287 */

	PARAM_HW_SIDETONE_FLAG = 288,	/* 288 */
	PARAM_HW_SIDETONE_GAIN,	/* 289 */
	PARAM_HW_SIDETONE_EQ,	/* 290 */

	PARAM_DSP_VOICE_VOLUME_TABLE,	/* 291 */
	PARAM_FM_RADIO_DIGITAL_VOLUME,	/* 292 */

	PARAM_REVERB_TIME_CONSTANT = 293,	/* 293 */
	PARAM_REVERB_LEVEL,	/* 294 */
	PARAM_SMART_COMPRESS_G1LIN,	/* 295 */
	PARAM_SMART_COMPRESS_G2T2,	/* 296 */
	PARAM_ECHO_PATH_CHANGE_DETECTION_THRESHOLD = 297,	/* 297 */

	PARAM_DL_NOISE_SUPP_MAX,	/* 298 */
	PARAM_DL_NOISE_SUPPRESSION_ENABLE,	/* 299 */

	PARAM_VOICE_MIC1_HPF_ENABLE,	/* 300 */
	PARAM_VOICE_MIC1_HPF_CUTOFF_FREQ,	/* 301 */
	PARAM_VOICE_MIC2_HPF_ENABLE,	/* 302 */
	PARAM_VOICE_MIC2_HPF_CUTOFF_FREQ,	/* 303 */

	PARAM_MULTIBAND_COMP_G_LOW = 308, /* 308 */
	PARAM_MULTIBAND_COMP_G_MID,	/* 309  */
	PARAM_MULTIBAND_COMP_G_HIGH,	/* 310  */

	PARAM_MULTIBAND_COMP_P_LOW,	/* 311 */
	PARAM_MULTIBAND_COMP_P_MID,	/* 312 */
	PARAM_MULTIBAND_COMP_P_HIGH,	/* 313 */

	PARAM_MULTIBAND_COMP_ALPHA_LOW,	/* 314 */
	PARAM_MULTIBAND_COMP_ALPHA_MID,	/* 315 */
	PARAM_MULTIBAND_COMP_ALPHA_HIGH,	/* 316 */

	PARAM_MULTIBAND_COMP_BETA_LOW,	/* 317 */
	PARAM_MULTIBAND_COMP_BETA_MID,	/* 318 */
	PARAM_MULTIBAND_COMP_BETA_HIGH,	/* 319 */
	PARAM_VOICE_LTPF_ENABLE,	/* 320 */

	PARAM_MULTIBAND_COMP_G,	/* 321  */
	PARAM_MULTIBAND_COMP_P,	/* 322  */
	PARAM_MULTIBAND_COMP_DT_P,	/* 323  */
	PARAM_MULTIBAND_COMP_ALPHA,	/* 324  */
	PARAM_MULTIBAND_COMP_BETA,	/* 325  */

	PARAM_MIC1_EQ_ENABLE, /*326  */
	PARAM_MIC2_EQ_ENABLE, /*327  */
	PARAM_SPEAKER_EQ_ENABLE, /*328  */

	PARAM_NLP_DL_LOOKAHEAD_ENABLE, /*329  */

	AUDIO_PARM_NUMBER	/* 330 */
}AudioParam_t;


#endif   /*_INC_AUDIO_TUNING_H_*/

